/* BlinkenLib
 * version 0.0.0 date 2007-02-12
 * Copyright 2007 Dirk Pannenbecker <dp@sd-gp.de>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 * a blinkenarea.org project
 */

/* How to compile:
 * gcc -W -Wall -O2 -o Blinken2mbs blinken2mbs.c ~/BlinkenLib-0.5.2_2006-05-10/BlinkenLib.a -I ~/BlinkenLib-0.5.2_2006-05-10/
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "BlinkenLib.h"

/* prototyp */
int BlinkenMoviePrintMBR( stBlinkenMovie * pMovie );
int BlinkenMovieSaveMBR( stBlinkenMovie * pMovie, char * pFilename );

int main( int argCnt, char * * args )
{
  stBlinkenMovie * pMovie;
  int i;
  char * str;

  //print info
  printf( "BlinkenLib - Blinken2mbr\n"
          "version 0.0.1 date 2007-02-12\n"
          "Copyright 2007 Dirk Pannenbecker <dp@sd-gp.de>\n"
          "Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html\n"
          "a blinkenarea.org project\n\n" );

  //print syntax
  if( argCnt <= 1 )
  {
    printf( "syntax: %s <parameter> [...]\n\n"
            "parameters:\n"
            "  -i <file>\n"
            "     read movie from file (*.blm, *.bmm, *.bml, *.bbm)\n"
            "  -p\n"
            "     print movie\n"
            "  -o <file>\n"
            "     write movie to file (*.asm)\n\n",
            args[0] );
    return 0;
  }

  //no movie loaded
  pMovie = NULL;

  //process parameters
  for( i = 1; i < argCnt; i++ )
  {

    //read movie
    if( strcmp( args[i], "-i" ) == 0 )
    {
      if( i + 1 < argCnt )
      {
        i++;
        if( pMovie != NULL )
          BlinkenMovieFree( pMovie );
        pMovie = BlinkenMovieLoad( args[i] );

        if( pMovie == NULL )
          printf( "could not read movie \"%s\"\n", args[i] );
        else
          printf( "movie \"%s\" read\n", args[i] );
      }
      else
        printf( "missing input filename for \"-i\"\n" );
    }

    //print movie
    else if( strcmp( args[i], "-p" ) == 0 )
    {
      if( pMovie == NULL )
        printf( "no movie loaded to print\n" );
      else
      {
        str = BlinkenMovieToString( pMovie );
        if( str == NULL )
          printf( "could not print movie\n" );
        else
        {
          printf( "%s\n", str );
          free( str );
          BlinkenMoviePrintMBR( pMovie );
        }
      }
    }


    //write asm-movie
    else if( strcmp( args[i], "-o" ) == 0 )
    {
      if( i + 1 < argCnt )
      {
        i++;
        if( pMovie == NULL )
          printf( "no movie loaded to write to \"%s\"\n", args[i] );
        else
          if( BlinkenMovieSaveMBR( pMovie, args[i] ) < 0 )
            printf( "could not write movie \"%s\"\n", args[i] );
          else
            printf( "movie \"%s\" written\n", args[i] );
      }
      else
        printf( "missing output filename for \"-o\"\n" );
    }


    //unknown parameter
    else
      printf( "unknown parameter \"%s\", call without parameters to get help\n", args[i] );

  }

  //free movie
  if( pMovie != NULL )
    BlinkenMovieFree( pMovie );

  return 0;
}


int BlinkenMoviePrintMBR( stBlinkenMovie * pMovie )
{
  stBlinkenMovie * pOutMovie;
  stBlinkenFrame * pFrame;
  int i, y, x, pFrameCount;

  if( pMovie == NULL )
    return -1;

  //convert movie to suitable format
  pOutMovie = BlinkenMovieClone( pMovie );
  if( pOutMovie == NULL )
    return -1;
  BlinkenMovieResize( pOutMovie, 8, 18, 1, 0 );
  printf( "movie resized to %ux%u-%u/%u\n", 8, 18, 1, 1 );

  pFrameCount=BlinkenMovieGetFrameCnt( pOutMovie );
  for( i = 0; i < pFrameCount; i++ )
  {
    pFrame = BlinkenFrameClone( BlinkenMovieGetFrame( pOutMovie ,i ) );
    if( BlinkenMovieAppendFrame( pOutMovie, pFrame ) != 0 )
      BlinkenFrameFree( pFrame );
    printf(".db 0b");
    for (y=8;y>=0;y--)
    {
      for (x=0;x<16;x+=2)
        printf("%d", BlinkenFrameGetPixel( pFrame, y, x, 0 ) );
      printf(", 0b");
      for (x=1;x<17;x+=2)
        printf("%d", BlinkenFrameGetPixel( pFrame, y, x, 0 ) );
      if (y != 0)
        printf(", 0b");
    }
    printf("\n");
  }

  //free copied movie
  BlinkenMovieFree( pOutMovie );

  return 0;
}

int BlinkenMovieSaveMBR( stBlinkenMovie * pMovie, char * pFilename )
{
  stBlinkenMovie * pOutMovie;
  stBlinkenFrame * pFrame;
  FILE * pFile;
  int i, y, x, pFrameCount;

  if( pMovie == NULL || pFilename == NULL )
    return -1;

  //convert movie to suitable format
  pOutMovie = BlinkenMovieClone( pMovie );
  if( pOutMovie == NULL )
    return -1;
  BlinkenMovieResize( pOutMovie, 8, 18, 1, 0 );
  printf( "movie resized to %ux%u-%u/%u\n", 8, 18, 1, 1 );
  //open file
  pFile = fopen( pFilename, "wt" );
  if( pFile == NULL )
  {
    BlinkenMovieFree( pOutMovie );
    return -1;
  }
  pFrameCount=BlinkenMovieGetFrameCnt( pOutMovie );
  for( i = 0; i < pFrameCount; i++ )
  {
    pFrame = BlinkenFrameClone( BlinkenMovieGetFrame( pOutMovie ,i ) );
    if( BlinkenMovieAppendFrame( pOutMovie, pFrame ) != 0 )
      BlinkenFrameFree( pFrame );

    int grade;
    int bytenr;
    int bitnr;

    fprintf( pFile, ".db 0b");
    for (grade=1;grade>-1;grade--) {
        y=7; //mit unterster zeile beginnen
        x=grade; //links beginnen, bei 0 oder 1
        for (bytenr=0;bytenr<9;bytenr++) {
            for (bitnr=7;bitnr>=0;bitnr--) {
                fprintf( pFile, "%d", BlinkenFrameGetPixel( pFrame, y, x, 0 ) );
                x=x+2;
                if (x>17) {
                    x=grade; //links beginnen, bei 0 oder 1
                    y--; //eine zeile weiter oben
                }
            }
            if (bytenr==8 && grade==0) fprintf( pFile, "\n"); else fprintf( pFile, ", 0b");
        }
        y--;
    }
    int duration=BlinkenFrameGetDuration(pFrame);
    fprintf( pFile, ".db %d, %d\n", ((int)((duration&0xFF00)>>8)), ((int)duration&255));
    fprintf( pFile, "\n"); //eine leerzeile zwischen zwei frames macht das ganze übersichtlicher

    /*for (y=8;y>=0;y--)
    {
      for (x=0;x<16;x+=2)
        fprintf( pFile, "%d", BlinkenFrameGetPixel( pFrame, y, x, 0 ) );
      fprintf( pFile, ", 0b");
      for (x=1;x<17;x+=2)
        fprintf( pFile, "%d", BlinkenFrameGetPixel( pFrame, y, x, 0 ) );
      if (y != 0)
        fprintf( pFile, ", 0b");
    }
    fprintf( pFile, "\n");*/
  }

  //close file
  fclose( pFile );

  //free copied movie
  BlinkenMovieFree( pOutMovie );

  //success

//  printf("\n%s\n",pFilename);
  return 0;
}
